XAI_MODELS = [
    {'model': 'grok-3-beta', 'endpoint': 'https://api.x.ai/v1/chat/completions', 'tokens': 8192},
    {'model': 'grok-3-fast-beta', 'endpoint': 'https://api.x.ai/v1/chat/completions', 'tokens': 8192},
    {'model': 'grok-3-mini-beta', 'endpoint': 'https://api.x.ai/v1/chat/completions', 'tokens': 8192},
    {'model': 'grok-3-mini-fast-beta', 'endpoint': 'https://api.x.ai/v1/chat/completions', 'tokens': 8192},
    {'model': 'grok-2-1212', 'endpoint': 'https://api.x.ai/v1/chat/completions', 'tokens': 8192},
    {'model': 'grok-2-vision-1212', 'endpoint': 'https://api.x.ai/v1/chat/completions', 'tokens': 8192},
    {'model': 'v1', 'endpoint': 'https://api.x.ai/v1/embeddings', 'tokens': None},
    {'model': 'grok-2-image', 'endpoint': 'https://api.x.ai/v1/images/generations', 'tokens': None}
]

OPENAI_MODELS = [{'model': 'whisper-1', 'endpoint': 'https://api.openai.com/v1/audio/transcriptions', 'tokens': None},
                    {'model': 'gpt-4o', 'endpoint': 'https://api.openai.com/v1/chat/completions', 'tokens': 4097},
                       {'model': 'gpt-4', 'endpoint': 'https://api.openai.com/v1/chat/completions', 'tokens': 8192},
                       {'model': 'gpt-4-0613', 'endpoint': 'https://api.openai.com/v1/chat/completions', 'tokens': 8192},
                       {'model': 'gpt-4-32k', 'endpoint': 'https://api.openai.com/v1/chat/completions', 'tokens': 32768},
                       {'model': 'gpt-4-32k-0613', 'endpoint': 'https://api.openai.com/v1/chat/completions', 'tokens': 32768},
                       {'model': 'gpt-3.5-turbo', 'endpoint': 'https://api.openai.com/v1/chat/completions', 'tokens': 8001},
                       {'model': 'gpt-3.5-turbo-0613', 'endpoint': 'https://api.openai.com/v1/chat/completions', 'tokens': 4097},
                       {'model': 'gpt-3.5-turbo-16k', 'endpoint': 'https://api.openai.com/v1/chat/completions', 'tokens': 16385},
                       {'model': 'gpt-3.5-turbo-16k-0613', 'endpoint': 'https://api.openai.com/v1/chat/completions', 'tokens': 16385},
                       {'model': 'gpt-3.5-turbo-instruct', 'endpoint': 'https://api.openai.com/v1/chat/completions', 'tokens': 4097},
                       {'model': 'babbage-002', 'endpoint': 'https://api.openai.com/v1/chat/completions', 'tokens': 16384},
                       {'model': 'davinci-002', 'endpoint': 'https://api.openai.com/v1/chat/completions', 'tokens': 16384},
                       {'model': 'text-embedding-ada-002', 'endpoint': 'https://api.openai.com/v1/embeddings', 'tokens': None},
                       {'model': 'gpt-3.5-turbo', 'endpoint': 'https://api.openai.com/v1/fine_tuning/jobs', 'tokens': 8001},
                       {'model': 'babbage-002', 'endpoint': 'https://api.openai.com/v1/fine_tuning/jobs', 'tokens': 16384},
                       {'model': 'davinci-002', 'endpoint': 'https://api.openai.com/v1/fine_tuning/jobs', 'tokens': 16384},
                       {'model': 'text-moderation-stable', 'endpoint': 'https://api.openai.com/v1/moderations', 'tokens': 2049},
                       {'model': 'text-moderation-latest', 'endpoint': 'https://api.openai.com/v1/moderations', 'tokens': 2049}]



import os
import PySimpleGUI as sg
from abstract_gui import *
from .gui_utils import *
from abstract_utilities import get_closest_match_from_list
from abstract_utilities.list_utils import ensure_nested_list,find_original_case
from abstract_utilities.class_utils import get_all_functions_for_instance,get_all_params,get_fun
class GuiUtilsManager:
    def __init__(self):
        # Mapping between Python PySimpleGUI components and their HTML/JS counterparts
        self.component_mappings = {
            "Button": lambda args: f'<button>{args.get("button_text", "Button")}</button>',
            "InputText": lambda args: f'<input type="text" value="{args.get("default_text", "")}" style="background-color:{args.get("background_color", "white")};" />',
            "Input": lambda args: f'<input type="text" value="{args.get("default_text", "")}" />',
            "ProgressBar": lambda args: f'<progress max="{args.get("max_value", 100)}" value="{args.get("default_value", 0)}"></progress>',
            "Frame": lambda args: f'<fieldset><legend>{args.get("title", "")}</legend>{args.get("layout", "")}</fieldset>'
            # Add other component mappings as needed
        }

    def make_component(self, component_type, *args, output_language='python', **kwargs):
        if output_language == 'javascript':
            if component_type in self.component_mappings:
                return self.component_mappings[component_type](kwargs)
            else:
                raise ValueError(f"Component {component_type} not supported for JavaScript.")
        
        # Else return Python PySimpleGUI component
        return self.make_pysimplegui_component(component_type, *args, **kwargs)

    def make_pysimplegui_component(self, component_type, *args, **kwargs):
        # Example: Mapping to PySimpleGUI components
        if component_type == 'Button':
            return sg.Button(*args, **kwargs)
        elif component_type == 'InputText':
            return sg.InputText(*args, **kwargs)
        elif component_type == 'ProgressBar':
            return sg.ProgressBar(*args, **kwargs)
        elif component_type == 'Frame':
            return sg.Frame(*args, **kwargs)
        # Add more PySimpleGUI component mappings as needed



def get_progress_frame()->list:
    return [
        [
            make_component("Frame", 'PROGRESS', layout=[
                [
                    make_component("InputText", 'Awaiting Prompt', key='-PROGRESS_TEXT-', background_color="light blue", auto_size_text=True, size=(20, 20)),
                    make_component("ProgressBar", 100, orientation='h', size=(10, 20), key='-PROGRESS-'),
                    make_component("Input", default_text='0', key=text_to_key("query count"), size=(30, 20), disabled=True, enable_events=True)
                ]
            ]),
            make_component("Frame", 'query title', layout=[
                [
                    make_component("Input", default_text="title of prompt", size=(30, 1), key=text_to_key('title input'))
                ]
            ]),
            make_component('Frame', "response nav", [
                get_left_right_nav(name='response text',section=False,push=False)
            ])
        ]
    ]
####submit options
def get_output_options()->list:
    return [
        [
         make_component("Button",button_text="SUBMIT QUERY",key="-SUBMIT_QUERY-", disabled=False,enable_evete=True),
         make_component("Button",button_text="CLEAR REQUESTS",key='-CLEAR_REQUESTS-', disabled=False,enable_evete=True),
         make_component("Button",button_text="CLEAR CHUNKS",key='-CLEAR_CHUNKS-', disabled=False,enable_evete=True),
         make_component("Button",button_text="GEN README",key='-GENERATE_README-', disabled=False,enable_evete=True)]
    ]




import os
from abstract_utilities import get_closest_match_from_list
from abstract_utilities.list_utils import ensure_nested_list,find_original_case
from abstract_utilities.class_utils import get_all_functions_for_instance,get_all_params,get_fun
from abstract_gui import make_component,AbstractWindowManager,AbstractBrowser,text_to_key,ensure_nested_list,expandable,RightClickManager,get_screen_dimensions
right_click_mgr=RightClickManager()
def get_standard_screen_dimensions(width=0.70,height=0.80):
    return get_screen_dimensions(width=width,height=height)
window_width,window_height=get_standard_screen_dimensions()
def get_left_right_nav(name,section=True,push=True):
    insert = f"{name} {'section ' if section else ''}"
    nav_bar = [make_component("Button",button_text="<-",key=text_to_key(f"{insert}back"),enable_events=True),
         make_component("input",default_text='0',key=text_to_key(f"{insert}number"),size=(4,1)),
         make_component("Button",button_text="->",key=text_to_key(f"{insert}forward"),enable_events=True)]
    if push:
        nav_bar=[make_component("Push")]+nav_bar+[make_component("Push")]
    return nav_bar
def generate_bool_text(title:str,args:dict={})->object:
    return make_component("Frame",title, layout=[[get_right_click_multi(key=text_to_key(text=title,section='text'),args=args)]],**expandable())
def get_tab_layout(title:str,layout:list=None)->object:
    if not layout:
        layout = get_right_click_multi(key=text_to_key(title),args={**expandable(size=(None, 5))})
    return make_component("Tab",title.upper(),ensure_nested_list(layout))
def get_column(layout:list,args:dict={})->object:
    return make_component("Column",ensure_nested_list(layout),**args)
def get_tab_group(grouped_tabs:list,args:dict={})->object:
    return make_component("TabGroup",ensure_nested_list(grouped_tabs),**args)
def get_right_click_multi(key:str,args:dict={})->object:
    return make_component("Multiline",**args,right_click_menu=right_click_mgr.get_right_click(key=key),key=key)
completion_token_keys = ['completion tokens available','completion tokens desired','completion tokens used']
prompt_token_keys = ['prompt tokens available','prompt tokens desired','prompt tokens used']
chunk_data_keys=['max chunk size','chunk length','chunk total']
all_token_keys = completion_token_keys+prompt_token_keys+chunk_data_keys+['chunk sectioned data']
response_types = ['instruction', 'json', 'bash', 'text']
model_type_keys=['role','response type']
percentage_keys = ['prompt percentage','completion percentage']
file_options_keys = ['auto chunk title','reuse chunk data','append chunks','scan mode all']
test_options_keys = ['test run','test files','test file input','test browse']
instructions_keys = ["instructions","additional_responses","suggestions","abort","database_query","notation","generate_title","additional_instruction","request_chunks","prompt_as_previous","token_adjustment"]
api_keys = ["header",'api key','api env']



from .routes import Flask,clipit_bp,jsonify,get_bp,request,abort,send_from_directory,CORS
def abstract_clip_app():
    app = Flask(__name__)
    CORS(app)

    # Register blueprint
    app.register_blueprint(clipit_bp)

    # Serve the HTML at “/” if localhost, else 403
    @app.route('/', methods=['GET'])
    def index():
        remote = request.remote_addr
        if remote not in ('127.0.0.1', '::1'):
            abort(403)

        html_dir = os.path.join(os.path.dirname(__file__), 'html')
        return send_from_directory(html_dir, 'drop-n-copy.html')

    # Error handlers
    @app.errorhandler(404)
    def not_found(e):
        return {'error': 'Not found'}, 404

    @app.errorhandler(403)
    def forbidden(e):
        return {'error': 'Forbidden'}, 403

    return app
